import { InvalidURLProvided } from "../errors.js";
export function processUrl(url, replaceProtocols) {
    const { protocol, host } = matchUrl(url);
    const allowedProtocols = Object.entries(replaceProtocols)
        .flat()
        .map((a) => a.toLowerCase());
    if (!allowedProtocols.includes(protocol))
        throw new InvalidURLProvided();
    return constructUrl(protocol, host, replaceProtocols);
}
function matchUrl(url) {
    const pattern = /^([^/:]+):\/\/([^/]+)/;
    const matched = url.match(pattern);
    if (!matched)
        throw new InvalidURLProvided();
    const [protocol, host] = [...matched]
        .slice(1, 3)
        .map((a) => a.toLowerCase());
    return { protocol, host };
}
function constructUrl(protocol, host, replaceProtocols) {
    protocol = protocol in replaceProtocols
        ? replaceProtocols[protocol]
        : protocol;
    return `${protocol}://${host}`;
}
//# sourceMappingURL=processUrl.js.map