"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processUrl = void 0;
const errors_js_1 = require("../errors.js");
function processUrl(url, replaceProtocols) {
    const { protocol, host } = matchUrl(url);
    const allowedProtocols = Object.entries(replaceProtocols)
        .flat()
        .map((a) => a.toLowerCase());
    if (!allowedProtocols.includes(protocol))
        throw new errors_js_1.InvalidURLProvided();
    return constructUrl(protocol, host, replaceProtocols);
}
exports.processUrl = processUrl;
function matchUrl(url) {
    const pattern = /^([^/:]+):\/\/([^/]+)/;
    const matched = url.match(pattern);
    if (!matched)
        throw new errors_js_1.InvalidURLProvided();
    const [protocol, host] = [...matched]
        .slice(1, 3)
        .map((a) => a.toLowerCase());
    return { protocol, host };
}
function constructUrl(protocol, host, replaceProtocols) {
    protocol = protocol in replaceProtocols
        ? replaceProtocols[protocol]
        : protocol;
    return `${protocol}://${host}`;
}
//# sourceMappingURL=processUrl.js.map